#include <Wire.h>
#include <Adafruit_Sensor.h>
#include <Adafruit_ADXL345_U.h>
#include <Servo.h>
#include "SoftwareSerial.h"
#include "DFRobotDFPlayerMini.h"


Adafruit_ADXL345_Unified accel = Adafruit_ADXL345_Unified();

//Question List
//Do you like to wear white shirt in black sweater?
//Do you prefer to wait silently?
//Do you like dogs?
//Do you understand?
//Do you brush your tooth before breakfast?
//Are you a morning person?
//NI XIHUAN WO MA?

// Use pins 2 and 3 to communicate with DFPlayer Mini
static const uint8_t PIN_MP3_TX = 2; // Connects to module's RX 
static const uint8_t PIN_MP3_RX = 3; // Connects to module's TX 
SoftwareSerial softwareSerial(PIN_MP3_RX, PIN_MP3_TX);

// Create the Player object
DFRobotDFPlayerMini player;


// Servos pins
const int frontServoPin = 11;
const int rightServoPin = 10;
const int backServoPin = 6; 
const int leftServoPin = 5;
Servo frontServo;
Servo rightServo;
Servo backServo;
Servo leftServo;

const int buttonPin = 7;
int buttonState = 0;

int behaviour;
int yes_count;
int no_count;
int shift = 12;

//Serial.println("start sense");
unsigned long startTime;
const unsigned long period = 5000;


void setup() {
  Serial.begin(9600);
  softwareSerial.begin(9600);

  pinMode(buttonPin, INPUT);

  if (player.begin(softwareSerial)) {
    player.volume(30);
  } else {
    Serial.println("Connecting to DFPlayer Mini failed!");
  }

  accel.begin();

  //Set random seed
  randomSeed(analogRead(180));

  // Servo set up
  frontServo.attach(frontServoPin);
  rightServo.attach(rightServoPin);
  backServo.attach(backServoPin);
  leftServo.attach(leftServoPin);
  
  startTime = millis();
}



void loop() {

  buttonState = digitalRead(buttonPin);

  int numbers[7] = {1,2,3,4,5,6,7};

  frontServo.write(0);
  rightServo.write(0);
  backServo.write(0);
  leftServo.write(0);

  behaviour = sense();

  // Jump to start //
  int count_start = 0;
  while (behaviour == 0) {
    while(count_start % 200 == 0){
      Serial.println("Hello hello, you are going to a party! Jump to start!");
      player.play(1);//****************************************************************
      say_hello();
      count_start += 1;
    }
    count_start += 1;
    behaviour = sense();
  }

  delay(2000);

  // Start //
  if (behaviour == 3){
    Serial.println("Here you need to answer some question. Noding your head to say yes. Are you ready?");
    player.play(2);//****************************************************************

    // Demonstrate YES //
    say_yes();
    int yes_behaviour = sense();
    delay(6000);

    // Check YES //
    int count_yes = 0;
    while (yes_behaviour != 2){
      while(count_yes % 100 == 0){
        Serial.println("I cant feel you? Are you ready?");
        player.play(3);//****************************************************************
        say_yes();
        count_yes+=1;
      }
      count_yes += 1;
      yes_behaviour = sense();
    }
    delay(2000);

    // Demostrate NO //
    Serial.println("Moving your head leaf and right to say no.");   // <-------------------------------------Audio is not right
    player.play(4);//****************************************************************
    say_no();
    int no_behaviour = sense();
    delay(3000);

    // Check NO //
    int count_no=0;
    while (no_behaviour != 1){
      while (count_no % 100 == 0){
        Serial.println("Come on! I can't feel you. Try to do it harder!");
        player.play(5);//****************************************************************
        say_no();
        count_no += 1;
      }
      count_no+=1;
      no_behaviour = sense();
    }
    delay(2000);

    // Ask Question //
    int q_count = 0;
    while (q_count<3){

      // Choose the random question
      int randomNumber = random(7);
      int index = numbers[randomNumber];
      while (index>22){
        randomNumber = random(7);
        index = numbers[randomNumber];
      }
      numbers[randomNumber]=25;

      // Play question
      Serial.println(index+shift);
      player.play(index+shift);//****************************************************************
      say_hello();
      int behaviour_q = sense();
      delay(4000);

      // Wait for answering question
      int count_q = 0;
      while (behaviour_q==0 || behaviour_q == 3){
        while(count_q % 150 == 0){
          Serial.println("Answer the question!");
          player.play(index+shift);//****************************************************************
          say_hello();
          count_q += 1;
        }
        count_q += 1;
        behaviour_q = sense();
      }

      // Record yes or no
      if (behaviour_q==2){
        yes_count+=1;
      }
      q_count+=1;
      delay(2000);
    }

    // Dancing //
    Serial.println("Now move your head with music, this will tell a lot about you.");
    player.play(6);//****************************************************************
    say_hello();
    delay(2000);
    
    // Play Music //
    Serial.println("Lalalalalalalalalalalalalalalalalalalalala.");
    player.play(7);//****************************************************************
    say_hello();
    delay(3000);

    // Sense Music Movement //
    int behaviour_dance = yes_count * dance();
    Serial.println(behaviour_dance);

    // Wait Music to end //
    delay(10000);

    // Categray //
    if (behaviour_dance <= 6){
      Serial.println("Yeah I notice, you should bring Axel to party.");
      player.play(8);//****************************************************************
      delay(10000);
    }else if (behaviour_dance > 6 && behaviour_dance <= 12){
      Serial.println("I see, you should invite Fabian to your party.");
      player.play(9);//****************************************************************
      delay(10000);
    }else if (behaviour_dance > 12 && behaviour_dance <= 18){
      Serial.println("Well done, Anahi will be perfect for you.");
      player.play(10);//****************************************************************
      delay(10000);
    }else if (behaviour_dance > 18 && behaviour_dance <= 24){
      Serial.println("Oh yes, that is a perfect dance, Anni will match you.");
      player.play(11);//****************************************************************
      delay(10000);
    }else{
      Serial.println("Amazing how you move, you might take Ekin to your party.");
      player.play(12);//****************************************************************
      delay(10000);
    }
  }

}


int findMax(int arr[], int size) {
  int max_value = arr[0];
  for (int i = 1; i < size; i++) {
    if (arr[i] > max_value) {
      max_value = arr[i];
    }
  }
  return max_value;
}

int findMin(int arr[], int size) {
  int min_value = arr[0];
  for (int i = 1; i < size; i++) {
    if (arr[i] < min_value) {
      min_value = arr[i];
    }
  }
  return min_value;
}

void say_hello(){
  //Serial.println("Hello");
  frontServo.write(180);
  rightServo.write(180);
  backServo.write(180);
  leftServo.write(180);
  delay(1000);
  frontServo.write(0);
  rightServo.write(0);
  backServo.write(0);
  leftServo.write(0);
  delay(1000);
  frontServo.write(180);
  rightServo.write(180);
  backServo.write(180);
  leftServo.write(180);
  delay(1000);
  frontServo.write(0);
  rightServo.write(0);
  backServo.write(0);
  leftServo.write(0);
}

void say_yes(){
  //Serial.println("Yes");
  frontServo.write(180);
  rightServo.write(0);
  backServo.write(0);
  leftServo.write(0);
  delay(1000);
  frontServo.write(0);
  rightServo.write(0);
  backServo.write(180);
  leftServo.write(0);
  delay(1000);
  frontServo.write(180);
  rightServo.write(0);
  backServo.write(0);
  leftServo.write(0);
  delay(1000);
  frontServo.write(0);
  rightServo.write(0);
  backServo.write(180);
  leftServo.write(0);
}

void say_no(){
  //Serial.println("No");
  frontServo.write(0);
  rightServo.write(180);
  backServo.write(0);
  leftServo.write(0);
  delay(1000);
  frontServo.write(0);
  rightServo.write(0);
  backServo.write(0);
  leftServo.write(180);
  delay(1000);
  frontServo.write(0);
  rightServo.write(180);
  backServo.write(0);
  leftServo.write(0);
  delay(1000);
  frontServo.write(0);
  rightServo.write(0);
  backServo.write(0);
  leftServo.write(180);
}

int sense(){

  int currentIndex = 0;
  sensors_event_t event;
  // Arrays to store acceleration values
  const int arraySize = 50;
  int ax_values[arraySize];
  int ay_values[arraySize];
  int az_values[arraySize];

  // If within the period, store values in arrays
  while (currentIndex < arraySize) {
    // Read acceleration values 
    accel.getEvent(&event);
    int ax = event.acceleration.x;
    int ay = event.acceleration.y;
    int az = event.acceleration.z;

    ax_values[currentIndex] = ax;
    ay_values[currentIndex] = ay;
    az_values[currentIndex] = az;
    currentIndex++;
  }
  // Calculate domains
  int ax_domain = findMax(ax_values, currentIndex) - findMin(ax_values, currentIndex);
  int ay_domain = findMax(ay_values, currentIndex) - findMin(ay_values, currentIndex);
  int az_domain = findMax(az_values, currentIndex) - findMin(az_values, currentIndex);

  // Find the list with the largest domain
  int max_domain = max(ax_domain, max(ay_domain, az_domain));

  //Serial.println(max_domain);

  if (max_domain <= 2){
    
    //means none
    behaviour = 0;
  }else if(ax_domain == max_domain){
    // means no
    behaviour = 1;
  }else if(ay_domain == max_domain){
    //means yes
    behaviour = 2;
  }else{
    //means mmm
    behaviour = 3;
  }
  // Return
  //Serial.println(behaviour);
  return behaviour;
}

int dance(){

  int currentIndex = 0;
  sensors_event_t event;
  // Arrays to store acceleration values
  const int arraySize = 50;
  int ax_values[arraySize];
  int ay_values[arraySize];
  int az_values[arraySize];

  // If within the period, store values in arrays
  while (currentIndex < arraySize) {
    // Read acceleration values 
    accel.getEvent(&event);
    int ax = event.acceleration.x;
    int ay = event.acceleration.y;
    int az = event.acceleration.z;

    ax_values[currentIndex] = ax;
    ay_values[currentIndex] = ay;
    az_values[currentIndex] = az;
    currentIndex++;
  }
  // Calculate domains
  int ax_domain = findMax(ax_values, currentIndex) - findMin(ax_values, currentIndex);
  int ay_domain = findMax(ay_values, currentIndex) - findMin(ay_values, currentIndex);
  int az_domain = findMax(az_values, currentIndex) - findMin(az_values, currentIndex);

  int max_sum = ax_domain + ay_domain + az_domain;

  return max_sum;
}


